#include <stdlib.h>
#include <stdio.h>
#include <getopt.h>
#include <string.h>

#include "NodeMonitorHelpers.h"
#include "FileWatcher.h"
#include "DirectoryWatcher.h"

#define DOCS \
"Usage:
	watcher [-m<command>] [-c<command>] [-a<command>] [-d<command>]
	            [-d] <file>
or
	watcher [-m<command>] [-c<command>] [-a<command>] [-d<command>]
	            [-n<command>] [-r<command>] [-d] <directory>

Where:
	file        is a path to a file or symbolic link to watch
	directory   is a path to a directory to watch
	
Options:
	-v            Be verbose.
	-1            Exit after triggering on one event.
	              Default: trigger forever.

  For directories, symlinks, and files:
	-m<command>   Call command when node moves.
	                %1=old path
	                %2=new path
	-c<command>   Call command when node's size/date changes.
	                %1=path
	-a<command>   Call command when node's attribute changes.
	                %1=path
	                %2=attribute
	-d<command>   Call command when file has been deleted.
	              No further triggers will occur.
	                %1=path

  For directories only:
	-n<command>   Call command when new node arrives in directory.
	               %1=directory path
	               %2=new file name
	-r<command>   Call command when a node is removed from directory.
                  Deleted file name is unavailable!
                   %1=directory path 
"

bool sDoOnlyOne = false;
					
int main(int argc, char *const argv[]) {
	sNodeMonitorDebugging = false;

	char *move = NULL;
	char *stat = NULL;
	char *attribute = NULL;
	char *deleted = NULL;

	char *newfile = NULL;
	char *removefile = NULL;
	
	bool containsDirectoryCommands = false;

			
	char option;
	while ((option = getopt(argc, argv, "v1m:c:a:d:n:r:")) != EOF && option!='?') {
		switch (option) {
			case 'v':
				sNodeMonitorDebugging = true;
				break;
			
			case '1':
				sDoOnlyOne=true;
				break;
			
			case 'm':
				move = optarg;
				break;
												
			case 'c':
				stat = optarg;
				break;
												
			case 'a':
				attribute = optarg;
				break;
												
			case 'd':
				deleted = optarg;
				break;
												
			case 'n':
				newfile = optarg;
				containsDirectoryCommands = true;
				break;
												
			case 'r':
				removefile = optarg;
				containsDirectoryCommands = true;
				break;
		}
	}

	if (option=='?' || optind==argc) {
		printf("%s", DOCS);
		exit(1);
	}	

	if (optind!=argc-1) {
		printf("Please provide a path to one node to watch.  You gave %d.\n", argc-optind);
		exit(1);
	}	

	char *path = argv[optind];
	
	BEntry entry(path);
	if (!entry.Exists() || entry.InitCheck()!=B_NO_ERROR) {
		printf("The path given is not a file, symlink, or directory, or it does not exist.\n");
		exit(!entry.Exists() ? -1 : entry.InitCheck());
	}

	if (!entry.IsDirectory() && containsDirectoryCommands) {
		printf("WARNING: The path given is not a directory.\n         The directory commands given will be ignored.\n");
	}

	sem_id waitingSemaphore;
	waitingSemaphore = create_sem (0, "waiting for godot");
	
	if (entry.IsDirectory()) {
		BDirectory dir(path);
		new DirectoryWatcher(waitingSemaphore, dir, entry,
		                     move, stat, attribute, deleted,
		                     newfile, removefile);
	} else {
		new FileWatcher(waitingSemaphore, entry,
		                     move, stat, attribute, deleted);
	}
	
	acquire_sem(waitingSemaphore);
	exit(0);
}